declare const _default: "\n#ifdef USE_REFLECTION_PROBES\n  uniform sampler2D uReflectionMap;\n  uniform int uReflectionMapEncoding;\n\n  #define MAX_MIPMAP_LEVEL 5.0\n\n  vec3 getPrefilteredReflection(vec3 reflected, float roughness) {\n    float lod = pow(roughness, 2.0) * MAX_MIPMAP_LEVEL; // TODO: verify reflection probe blurring code\n    // float lod = pow(roughness, 1.5) * MAX_MIPMAP_LEVEL;\n    float upLod = floor(lod);\n    float downLod = ceil(lod);\n\n    vec3 a = decode(texture2D(uReflectionMap, envMapOctahedral(reflected, 0.0, upLod)), uReflectionMapEncoding).rgb;\n    vec3 b = decode(texture2D(uReflectionMap, envMapOctahedral(reflected, 0.0, downLod)), uReflectionMapEncoding).rgb;\n\n    return mix(a, b, lod - upLod);\n  }\n\n  vec3 EnvBRDFApprox( vec3 specularColor, float roughness, float NoV ) {\n    const vec4 c0 = vec4(-1.0, -0.0275, -0.572, 0.022 );\n    const vec4 c1 = vec4( 1.0, 0.0425, 1.04, -0.04 );\n    vec4 r = roughness * c0 + c1;\n    float a004 = min( r.x * r.x, exp2( -9.28 * NoV ) ) * r.x + r.y;\n    vec2 AB = vec2( -1.04, 1.04 ) * a004 + r.zw;\n    return specularColor * AB.x + AB.y;\n  }\n\n  #if defined(USE_CLEAR_COAT)\n    // https://google.github.io/filament/Filament.md.html#lighting/imagebasedlights/clearcoat\n    void evaluateClearCoatIBL(const PBRData data, float ao, inout vec3 Fd, inout vec3 Fr) {\n      #if defined(USE_NORMAL_MAP) || defined(USE_CLEAR_COAT_NORMAL_MAP)\n        float clearCoatNoV = abs(dot(data.clearCoatNormal, data.viewWorld)) + FLT_EPS;\n        vec3 clearCoatR = reflect(-data.viewWorld, data.clearCoatNormal);\n      #else\n        float clearCoatNoV = data.NdotV;\n        vec3 clearCoatR = data.reflectionWorld;\n      #endif\n      // The clear coat layer assumes an IOR of 1.5 (4% reflectance)\n      float Fc = F_Schlick(0.04, 1.0, clearCoatNoV) * data.clearCoat;\n      float attenuation = 1.0 - Fc;\n      Fr *= (attenuation * attenuation);\n      Fr += getPrefilteredReflection(clearCoatR, data.clearCoatRoughness) * (ao * Fc);\n      Fd *= attenuation;\n    }\n  #endif\n\n  void EvaluateLightProbe(inout PBRData data, float ao) {\n    // TODO: energyCompensation\n    float energyCompensation = 1.0;\n\n    vec3 diffuseIrradiance = getIrradiance(data.normalWorld, uReflectionMap, uReflectionMapEncoding);\n    vec3 Fd = data.diffuseColor * diffuseIrradiance * ao;\n\n    vec3 specularReflectance = EnvBRDFApprox(data.f0, data.roughness, data.NdotV);\n    vec3 prefilteredRadiance = getPrefilteredReflection(data.reflectionWorld, data.roughness);\n\n    vec3 Fr = specularReflectance * prefilteredRadiance * ao;\n    Fr *= energyCompensation;\n\n    // vec3 Fs = EvaluateSheen(data, NdotH, data.NdotV, NdotL);\n    vec3 sheenReflectance = EnvBRDFApprox(data.sheenColor, data.sheenRoughness, data.NdotV);\n    vec3 sheenRadiance = getPrefilteredReflection(data.reflectionWorld, data.sheenRoughness);\n    // vec3 Fs = sheenRadiance + sheenReflectance * ao;\n    vec3 Fs = vec3(0.0);\n    data.sheen += Fs;\n\n    #ifdef USE_CLEAR_COAT\n      evaluateClearCoatIBL(data, ao, Fd, Fr);\n    #endif\n\n    data.indirectDiffuse += Fd;\n    data.indirectSpecular += Fr;\n    data.indirectSpecular += Fs;\n  }\n#endif\n";
export default _default;
