declare const _default: "\n#ifdef USE_METALLIC_ROUGHNESS_WORKFLOW\n  uniform float uMetallic;\n  uniform float uRoughness;\n\n  // Source: Google/Filament/Overview/4.8.3.3 Roughness remapping and clamping, 07/2019\n  // Minimum roughness to avoid division by zerio when 1/a^2 and to limit specular aliasing\n  // This could be 0.045 when using single precision float fp32\n  #define MIN_ROUGHNESS 0.089\n\n  #ifdef USE_METALLIC_ROUGHNESS_MAP\n    // R = ?, G = roughness, B = metallic\n    uniform sampler2D uMetallicRoughnessMap;\n\n    #ifdef USE_METALLIC_ROUGHNESS_MAP_TEX_COORD_TRANSFORM\n      uniform mat3 uMetallicRoughnessMapTexCoordTransform;\n    #endif\n\n    // TODO: sampling the same texture twice\n    void getMetallic(inout PBRData data) {\n      #ifdef USE_METALLIC_ROUGHNESS_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, METALLIC_ROUGHNESS_MAP_TEX_COORD_INDEX, uMetallicRoughnessMapTexCoordTransform);\n      #else\n        vec2 texCoord = getTextureCoordinates(data, METALLIC_ROUGHNESS_MAP_TEX_COORD_INDEX);\n      #endif\n      vec4 texelColor = texture2D(uMetallicRoughnessMap, texCoord);\n      data.metallic = uMetallic * texelColor.b;\n      data.roughness = uRoughness * texelColor.g;\n    }\n\n    void getRoughness(inout PBRData data) {\n      // NOP, already read in getMetallic\n    }\n  #else\n    #ifdef USE_METALLIC_MAP\n      uniform sampler2D uMetallicMap; //assumes linear, TODO: check gltf\n\n      #ifdef USE_METALLIC_MAP_TEX_COORD_TRANSFORM\n        uniform mat3 uMetallicMapTexCoordTransform;\n      #endif\n\n      void getMetallic(inout PBRData data) {\n        #ifdef USE_METALLIC_MAP_TEX_COORD_TRANSFORM\n          vec2 texCoord = getTextureCoordinates(data, METALLIC_MAP_TEX_COORD_INDEX, uMetallicMapTexCoordTransform);\n        #else\n          vec2 texCoord = getTextureCoordinates(data, METALLIC_MAP_TEX_COORD_INDEX);\n        #endif\n        data.metallic = uMetallic * texture2D(uMetallicMap, texCoord).r;\n      }\n    #else\n      void getMetallic(inout PBRData data) {\n        data.metallic = uMetallic;\n      }\n    #endif\n\n    #ifdef USE_ROUGHNESS_MAP\n      uniform sampler2D uRoughnessMap; //assumes linear, TODO: check glTF\n\n      #ifdef USE_ROUGHNESS_MAP_TEX_COORD_TRANSFORM\n        uniform mat3 uRoughnessMapTexCoordTransform;\n      #endif\n\n      void getRoughness(inout PBRData data) {\n\n        #ifdef USE_ROUGHNESS_MAP_TEX_COORD_TRANSFORM\n          vec2 texCoord = getTextureCoordinates(data, ROUGHNESS_MAP_TEX_COORD_INDEX, uRoughnessMapTexCoordTransform);\n        #else\n          vec2 texCoord = getTextureCoordinates(data, ROUGHNESS_MAP_TEX_COORD_INDEX);\n        #endif\n        data.roughness = uRoughness * texture2D(uRoughnessMap, texCoord).r + 0.01;\n      }\n    #else\n      void getRoughness(inout PBRData data) {\n        data.roughness = uRoughness + 0.01;\n      }\n    #endif\n  #endif\n#endif\n";
export default _default;
